//	---------------------------------------------------------------------------
//	$Header: /fp/fp1/67NSYY1QMG4igynohtna/anthony/msgboxstream.h,v 1.1 2002/06/01 15:25:32 ynohtna Exp $
//
//	(c) Copyright 2002 Anthony Bowyer-Lowe.  All rights reserved.
//
//	Anthony Bowyer-Lowe / ynohtna
//	ynohtna@ynohtna.org
//	http://www.ynohtna.org/
//	---------------------------------------------------------------------------
#if !defined(STUFF_YOUR_MESSAGE_BOX_UP_YOUR_STREAM)
#define STUFF_YOUR_MESSAGE_BOX_UP_YOUR_STREAM

#include <iostream>
#include <ostream>
#include <sstream>


//	---------------------------------------------------------------------------
namespace anthony {
namespace msgbox {
#if 0
	}	//	Nil auto-indentum.
}
#endif


//	---------------------------------------------------------------------------
#if defined(WIN32)	//	Yay, Windows!

//	Win32 message box helper, specialised against char and wchar_t ------------
template<typename E>
struct w32_messagebox
{
	enum { default_style = MB_OK };
	int display(const E* title, const E* text, int style);
};

template<>
int w32_messagebox<char>::display(const char* title, const char* text, int style) {
	return MessageBoxA(0, text, title, style);
}

template<>
int w32_messagebox<wchar_t>::display(const wchar_t* title, const wchar_t* text, int style) {
	return MessageBoxW(0, text, title, style);
}

#define DEFAULT_MSGBOX_MECH	w32_messagebox

#else				//	Yay, not Windows!

#define DEFAULT_MSGBOX_MECH	cout_messagebox

#endif


//	Non-Win32 implementation of a message box - no use, merely an example -----
template<typename E>
struct cout_messagebox
{
	enum { default_style = 0 };
	int display(const E* title, const E* text, int style) {
		std::cout << '[' << title << ']' << std::endl;
		std::cout << text << std::endl;
		std::string input;
		std::cin >> input;
		return input[0] == 'y' || input[0] == 'Y';
	}
};


//	---------------------------------------------------------------------------
namespace helper
{	//	Manipulators for insertion into the msgboxstream to trigger operations.
	struct style_adder {
		int v_;
		style_adder(int v) : v_(v)
		{	}
	};
	struct display {
	};
}	//	namespace helpers


//	Note: both icon and buttons perform the same action - namely modifying
//	the style property of the message box. Both are supplied to provide a
//	modicum of readability (xref literate-programming) to client code. Viz:
//	mb << icon(my_icon_style) << buttons(my_desired_buttons);
inline helper::style_adder icon(int i) {
	return helper::style_adder(i);
}

inline helper::style_adder buttons(int i) {
	return helper::style_adder(i);
}

//	Allow client code to display the message box through an insertion.
//	Personally I think "mb << msgbox::display();" is ugly and involves too
//	much typing, but I've included this as an example of how to code different
//	custom insertion operations.
inline helper::display display() {
	return helper::display();
}


//	The actual msgboxstream class itself --------------------------------------
template<typename E, typename T = std::char_traits<E>, typename P = DEFAULT_MSGBOX_MECH<E> >
class basic_msgboxstream : public std::basic_ostream<E, T>
{
	typedef std::basic_ostream<E, T>	parent;	//	Typedefs are A Good Thing.
	typedef std::basic_streambuf<E, T>	buffer;
	typedef basic_msgboxstream<E, T, P>	my_type;

	P					box_mechanism_;	//	Object that knows how to display our msgbox.
	std::stringstream	buffer_;		//	Don't need custom streambufs for this class.
	std::stringstream	title_;
	int					style_;
	int					result_;

public:
	basic_msgboxstream()
		:	parent(buffer_.rdbuf()), box_mechanism_(P()), style_(P::default_style), result_(0)
	{	}
	basic_msgboxstream(P worker)
		:	parent(buffer_.rdbuf()), box_mechanism_(worker), style_(P::default_style), result_(0)
	{	}

	//	Returns the stream that contains the title to be displayed for this msgbox.
	std::ostream& title() {
		return title_;
	}

	//	Show
	int display() {
		flush();			//	Ensure we've collected all needed content.
		title_.flush();
		result_ = box_mechanism_.display(title_.str().c_str(), buffer_.str().c_str(), style_);
		return result_;
	}

	//	Accessor and mutator for the msgbox style.
	void style(int s) {
		style_ = s;
	}
	int style() const {
		return style_;
	}

	//	Accessors for the msgbox's result code. Is zero if it hasn't been displayed.
	operator int() const {
		return result_;
	}
	int result() const {
		return result_;
	}

	//	Manipulations triggered by insertion of custom objects.
	friend my_type& operator<<(my_type& mb, const helper::style_adder& sa) {
		mb.style_ |= sa.v_;
		return mb;
	}
	friend my_type& operator<<(my_type& mb, const helper::display&) {
		mb.display();
		return mb;
	}
};


//	For ease of use in 99% of the real world ----------------------------------
typedef basic_msgboxstream<char>	msgboxstream;
typedef basic_msgboxstream<wchar_t>	msgboxstreamw;


//	---------------------------------------------------------------------------
	}	//	namespace msgbox
}	//	namespace anthony


#endif	//	STUFF_YOUR_MESSAGE_BOX_UP_YOUR_STREAM
